# encoding: utf-8
# Copyright (C) 2016 John Törnblom


from xtuml import where_eq as where

from utils import RSLTestCase

class TestCreateDelete(RSLTestCase):

    def test_not_first_in_loop(self):
        self.metamodel.define_class('A', [('Name', 'string')])

        self.assertEqual(len(self.metamodel.select_many('A')), 0)
        self.eval_text('.create object instance a of A')
        self.assertEqual(len(self.metamodel.select_many('A')), 1)
        
        self.metamodel.new('A', Name='1')
        self.metamodel.new('A', Name='2')
        self.metamodel.new('A', Name='3')
        self.metamodel.new('A', Name='4')
        
        tmpl = '''
        .select any a from instances of A where (selected.Name == "%s")
        .delete object instance a
        '''
        
        self.eval_text(tmpl % '2')
        
        self.assertTrue(self.metamodel.select_any('A', where(Name='1')))
        self.assertFalse(self.metamodel.select_any('A', where(Name='2')))
        self.assertTrue(self.metamodel.select_any('A', where(Name='3')))
        self.assertTrue(self.metamodel.select_any('A', where(Name='4')))
        
        self.eval_text(tmpl % '4')
        
        self.assertTrue(self.metamodel.select_any('A', where(Name='1')))
        self.assertFalse(self.metamodel.select_any('A', where(Name='2')))
        self.assertTrue(self.metamodel.select_any('A', where(Name='3')))
        self.assertFalse(self.metamodel.select_any('A', where(Name='4')))
        
        self.eval_text(tmpl % '1')
        
        self.assertFalse(self.metamodel.select_any('A', where(Name='1')))
        self.assertFalse(self.metamodel.select_any('A', where(Name='2')))
        self.assertTrue(self.metamodel.select_any('A', where(Name='3')))
        self.assertFalse(self.metamodel.select_any('A', where(Name='4')))
        
        


# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring


class TestIf(RSLTestCase):

    @evaluate_docstring
    def test_single_if_true(self, rc):
        '''
        .if ( 0 == 0 )
            .exit 1
        .end if
        .exit 0
        '''
        self.assertEqual(1, rc)
        
    @evaluate_docstring
    def test_single_if_false(self, rc):
        '''
        .if ( 0 == 1 )
            .exit 0
        .end if
        .exit 1
        '''
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_if_with_comments(self, rc):
        '''
        .if ( 0 == 0 ) .//
            .exit 1    .//
        .elif (0 == 1) .//
            .exit 2    .//
        .end if        .//
        .exit 0
        '''
        self.assertEqual(1, rc)
        
    @evaluate_docstring
    def test_elif_first(self, rc):
        '''
        .assign x = 0
        .if (1 == 0)
            .assign x = 1
        .elif (1 == 1)
            .assign x = 2
        .else
            .assign x = 3
        .end if
        .exit x
        
        '''
        self.assertEqual(2, rc)
        
    @evaluate_docstring
    def test_elif_middle(self, rc):
        '''
        .if ( 0 == 1 )
            .exit 0
        .elif ( 0 == 2)
            .exit 1
        .elif (0 == 0)
            .exit 2
        .elif (0 == 3)
            .exit 3
        .else
            .exit 4
        .end if
        .exit 5
        '''
        self.assertEqual(2, rc)
        
    @evaluate_docstring
    def test_elif_last(self, rc):
        '''
        .if ( 0 == 1 )
            .exit 0
        .elif ( 0 == 2)
            .exit 1
        .elif (0 == 0)
            .exit 2
        .end if
        .exit 3
        '''
        self.assertEqual(2, rc)
        
    @evaluate_docstring
    def test_if_elif_else(self, rc):
        '''
        .if ( 0 == 1 )
            .exit 0
        .elif ( 0 == 2)
            .exit 1
        .elif (0 == 3)
            .exit 2
        .else
            .exit 3
        .end if
        .exit 4
        '''
        self.assertEqual(3, rc)
        
        



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2015 John Törnblom
'''
rule-specification language (RSL) interpreter.
'''

import sys
import logging
import os
import xtuml

import rsl.version
 

complete_usage = '''
USAGE: 

   %s  [-arch <string>] ... [-import <string>] ... [-include <string>] ... [-d <integer>] ... [-diff <string>] [-emit <string>] [-priority <integer>] [-lVHs] [-lSCs] [-l2b] [-l2s] [-l3b] [-l3s] [-nopersist] [-dumpsql <file>] [-force] [-integrity] [-e <string>] [-t <string>] [-v <string>] [-qim] [-q] [-l] [-f <string>] [-# <integer>] [//] [-version] [-h]


Where: 

   -arch <string>  (accepted multiple times)
     (value required)  Archetype file name(s)

   -import <string>  (accepted multiple times)
     (value required)  Data file name(s)

   -include <string>  (accepted multiple times)
     (value required) add a path to list of dirs to search for include files

   -d <integer>  (accepted multiple times)
     (value required)  The domain code.  This argument must immediately precede the "-import" argument that it applies to.

   -diff <string>
     (value required)  save a diff of all emits to a filename

   -priority <integer>
     (value required)  Set process priority.  Acceptable values are:

             NORMAL_PRIORITY_CLASS = 32

             IDLE_PRIORITY_CLASS = 64

             HIGH_PRIORITY_CLASS = 128

             REALTIME_PRIORITY_CLASS = 256

             BELOW_NORMAL_PRIORITY_CLASS = 16384 (default)

             ABOVE_NORMAL_PRIORITY_CLASS =
     32768

   -emit <string>
     (value required) Chose when to emit. Acceptable values are:

             never = never emit to disk

             change = only emit to disk when files differ (default)

             always = always emit to disk, even when the content in memory is the same as the content on disk

   -lVHs
     Use VHDL source license

   -lSCs
     Use SystemC source license

   -l2b
     Use MC-2020 binary license

   -l2s
     Use MC-2020 source license

   -l3b
     Use MC-3020 binary license

   -l3s
     Use MC-3020 source license

   -nopersist
     Disable persistence

   -dumpsql <file>
     (value required)  Dump the instance population as SQL insert statements

   -force
     make read-only emit files writable

    -integrity
     check the model for integrity violations upon program exit

   -e <string>
     (value required)  Enable specified feature

   -t <string>
     (value required)  Full-blast logging

   -v <string>
     (value required)  Verbose mode (STMT, COMP, or SYS)

   -qim
     Quiet insert mismatches. Do not print warnings if insert data doesn't populate all attributes.

   -q
     Quit on error

   -l
     Use log file

   -f <string>
     (value required)  Generated file name (database)

   -# <integer>
     (value required)  Number of files to generate

   //,  -ignore_rest
     Ignores the rest of the labeled arguments following this flag.

   -version
     Displays version information and exits.

   -h
     Displays usage information and exits.


   gen_erate
'''

brief_usage = '''
Brief USAGE: 
   %s  [-arch <string>] ... [-import <string>] ... [-include <string>] ... [-d <integer>] ... [-diff <string>] [-emit <string>] [-priority <integer>] [-lVHs] [-lSCs] [-l2b] [-l2s] [-l3b] [-l3s] [-nopersist] [-dumpsql <file>] [-force] [-integrity] [-e <string>] [-t <string>] [-v <string>] [-qim] [-q] [-l] [-f <string>] [-# <integer>] [//] [-version] [-h]

For complete USAGE and HELP type: 
   %s -h
'''


def main(argv=None):
    loglevel = logging.INFO
    database_filename = 'mcdbms.gen'
    enable_persistance = True
    dump_sql_file = ''
    force_overwrite = False
    emit_when = 'change'
    diff_filename = None
    inputs = list()
    includes = ['.']
    check_integrity = False
    argv = argv or sys.argv
    quiet_insert_mismatch = False
    
    i = 1
    while i < len(argv):
        if argv[i] == '-arch':
            i += 1
            inputs.append((argv[i], 'arc'))

        elif argv[i] == '-import':
            i += 1
            inputs.append((argv[i], 'sql'))

        elif argv[i] == '-include':
            i += 1
            includes.append(argv[i])

        elif argv[i] == '-emit':
            i += 1
            emit_when = argv[i]
    
        elif argv[i] == '-f':
            i += 1
            database_filename = argv[i]

        elif argv[i] == '-force':
            force_overwrite = True

        elif argv[i] == '-integrity':
            check_integrity = True
            
        elif argv[i] == '-diff':
            i += 1
            diff_filename = argv[i]
            
        elif argv[i] == '-nopersist':
            enable_persistance = False
            
        elif argv[i] == '-dumpsql':
            i += 1
            dump_sql_file = argv[i]

        elif argv[i] == '-v':
            i += 1
            loglevel = logging.DEBUG
            
        elif argv[i] == '-qim':
            quiet_insert_mismatch = True
            
        elif argv[i] == '-version':
            print(rsl.version.complete_string)
            sys.exit(0)
            
        elif argv[i] == '-h':
            print(complete_usage % argv[0])
            sys.exit(0)
            
        elif argv[i] in ['//', '-ignore_rest']:
            break

        # ignore these options
        elif argv[i] in ['-lVHs', '-lSCs', '-l2b', '-l2s', '-l3b', '-l3s',
                             '-q', '-l']:
            pass
            
        # ignore these options (which expects a following value)
        elif argv[i] in ['-d', '-priority', '-e', '-t', '-#']:
            i += 1
            
        else:
            print("PARSE ERROR: Argument: %s" % argv[i])
            print("Couldn't find match for argument")
            print(brief_usage % (argv[0], argv[0]))
            sys.exit(1)
            
        i += 1
        
    logging.basicConfig(stream=sys.stdout, level=loglevel)
    
    id_generator = xtuml.IntegerGenerator()
    metamodel = xtuml.MetaModel(id_generator)
    loader = xtuml.ModelLoader()
    
    if quiet_insert_mismatch:
        load_logger = logging.getLogger(xtuml.load.__name__)
        load_logger.setLevel(logging.ERROR)

    if diff_filename:
        with open(diff_filename, 'w') as f:
            f.write(' '.join(argv))
            f.write('\n')
            
    if enable_persistance and os.path.isfile(database_filename):
        loader.filename_input(database_filename)
        
    for filename, kind in inputs:
        if kind == 'sql':
            loader.filename_input(filename)
            
        elif kind == 'arc':
            loader.populate(metamodel)
            rt = rsl.Runtime(metamodel, emit_when, force_overwrite, diff_filename)
            ast = rsl.parse_file(filename)
            rsl.evaluate(rt, ast, includes)
            loader = xtuml.ModelLoader()
            
        else:
            #should not happen
            print("Unknown %s is of unknown kind '%s', skipping it" % (filename, kind))

    errors = 0
    if check_integrity:
        errors += xtuml.check_association_integrity(metamodel)
        errors += xtuml.check_uniqueness_constraint(metamodel)
        
    if enable_persistance:
        xtuml.persist_database(metamodel, database_filename)

    if dump_sql_file != '':
        xtuml.persist_instances(metamodel, dump_sql_file)

    return errors


if __name__ == '__main__':
    num_errors = main()
    sys.exit(num_errors > 0)



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

import os

from utils import RSLTestCase
from utils import evaluate_docstring

from rsl.runtime import RuntimeException


class TestIntrinsic(RSLTestCase):

    @evaluate_docstring
    def test_get_env_var(self, rc):
        '''
        .invoke rc = GET_ENV_VAR("PATH")
        .exit rc.result
        '''
        self.assertEqual(os.environ['PATH'], rc)

    @evaluate_docstring
    def test_get_env_var_failure(self, rc):
        '''
        .invoke rc = GET_ENV_VAR("UNKNOWN_PATH")
        .exit rc.success
        '''
        self.assertFalse(rc)

    @evaluate_docstring
    def test_put_env_var(self, rc):
        '''
        .invoke rc = PUT_ENV_VAR("MY_PATH", "test")
        .exit rc.success
        '''
        self.assertTrue(rc)
        self.assertEqual(os.environ["MY_PATH"], "test")
    
    @evaluate_docstring
    def test_file_read_write(self, rc):
        '''
        .invoke rc = FILE_WRITE("/tmp/RSLTestCase", "Hello world!")
        .if ( not rc.success )
            .exit ""
        .end if
        .invoke rc = FILE_READ("/tmp/RSLTestCase")
        .if ( not rc.success )
            .exit ""
        .end if
        .exit rc.result
        '''
        self.assertEqual(rc, "Hello world!\n")
    
    @evaluate_docstring
    def test_file_read_error(self, rc):
        '''
        .invoke rc = FILE_READ("/")
        .exit rc.success
        '''
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_file_write_error(self, rc):
        '''
        .invoke rc = FILE_WRITE("/", "TEST")
        .exit rc.success
        '''
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_shell_command_exit_0(self, rc):
        '''
        .invoke rc = SHELL_COMMAND("exit 0")
        .exit rc.result
        '''
        self.assertEqual(0, rc)
    
    @evaluate_docstring
    def test_shell_command_exit_1(self, rc):
        '''
        .invoke rc = SHELL_COMMAND("exit 1")
        .exit rc.result
        '''
        self.assertEqual(1, rc)
        
    @evaluate_docstring
    def test_integer_to_string(self, rc):
        '''
        .invoke rc = INTEGER_TO_STRING(1)
        .exit rc.result
        '''
        self.assertEqual("1", rc)
        
    @evaluate_docstring
    def test_real_to_string(self, rc):
        '''
        .invoke rc = REAL_TO_STRING(1.1)
        .exit rc.result
        '''
        self.assertEqual("1.1", rc)
    
    @evaluate_docstring
    def test_boolean_to_string(self, rc):
        '''
        .invoke rc = BOOLEAN_TO_STRING(False)
        .exit rc.result
        '''
        self.assertEqual("FALSE", rc)
    
    @evaluate_docstring
    def test_string_to_integer(self, rc):
        '''
        .invoke rc = STRING_TO_INTEGER("1")
        .exit rc.result
        '''
        self.assertEqual(1, rc)
        
    @evaluate_docstring
    def test_string_to_integer_invalid(self, rc):
        '''
        .invoke rc = STRING_TO_INTEGER("test")
        '''
        self.assertIsInstance(rc, RuntimeException)
        
    @evaluate_docstring
    def test_string_to_integer_with_spaces(self, rc):
        '''
        .invoke rc = STRING_TO_INTEGER(" 1 ")
        .exit rc.result
        '''
        self.assertEqual(1, rc)
        
    @evaluate_docstring
    def test_string_to_real(self, rc):
        '''
        .invoke rc = STRING_TO_REAL("1.1")
        .exit rc.result
        '''
        self.assertEqual(1.1, rc)
    
    @evaluate_docstring
    def test_string_to_real_invalid(self, rc):
        '''
        .invoke rc = STRING_TO_REAL("test")
        '''
        self.assertIsInstance(rc, RuntimeException)
        
    @evaluate_docstring
    def test_string_to_real_with_spaces(self, rc):
        '''
        .invoke rc = STRING_TO_REAL(" 1.1 ")
        .exit rc.result
        '''
        self.assertEqual(1.1, rc)




# encoding: utf-8
# Copyright (C) 2016 John Törnblom
import string

from utils import RSLTestCase
from utils import evaluate_docstring
from utils import expect_exception

from rsl.parse import ParseException



def buffer_docstring(f):
    tmpl = string.Template('''
        .function f
$value
        .end function
        .invoke res = f()
        .print "$${res.body}"
        .exit res.body
    ''')

    def wrapper(self):
        filename = f.__module__ + '.' + f.__name__
        text = tmpl.substitute(value=f.__doc__)
        rc = self.eval_text(text, filename)

        return f(self, rc)

    return wrapper


class TestLiteral(RSLTestCase):

    @expect_exception(ParseException)
    @buffer_docstring
    def test_dot(self, rc):
        '.'
        pass

    @buffer_docstring
    def test_dotdot(self, rc):
        '..'
        self.assertEqual(".\n", rc)

    @expect_exception(ParseException)
    @buffer_docstring
    def test_dollar(self, rc):
        '$'
        pass

    @buffer_docstring
    def test_dollardollar(self, rc):
        '$$'
        self.assertEqual("$\n", rc)

    @buffer_docstring
    def test_dotdot_comment(self, rc):
        '..comment'
        self.assertEqual(".comment\n", rc)





# encoding: utf-8
# Copyright (C) 2015 John Törnblom
import os
import time

from utils import RSLTestCase
from utils import evaluate_docstring


class TestEmit(RSLTestCase):

    @evaluate_docstring
    def test_emit_hello_world(self, rc):
        '''Hello world
        .emit to file "/tmp/RSLTestCase"'''
        
        with open("/tmp/RSLTestCase") as f:
            try:
                # python 3.x
                self.assertRegex(f.read(), "Hello world\n")
            except AttributeError:
                # python 2.7
                self.assertRegexpMatches(f.read(), "Hello world\n")
        
    def test_emit_without_linebreak_case1(self):
        text = 'Hello ' + '\\' + '\n' + 'world' + '\n' 
        text+= '.emit to file "/tmp/RSLTestCase"' 
        
        self.eval_text(text)
        
        with open("/tmp/RSLTestCase") as f:
            self.assertEqual(f.read(), "Hello world\n")
            
    def test_emit_without_linebreak_case2(self):
        text = 'Hello world' + '\\' + '\\' + '\\' + '\n' 
        text+= '.emit to file "/tmp/RSLTestCase"' 
        
        self.eval_text(text)
        
        with open("/tmp/RSLTestCase") as f:
            self.assertEqual(f.read(), "Hello world\\\n")
    
    def test_emit_escaped_backslash(self):
        text = 'Hello world' + '\\' + '\\' + '\n' 
        text+= 'Test\n'
        text+= '.emit to file "/tmp/RSLTestCase"' 
        
        self.eval_text(text)
        
        with open("/tmp/RSLTestCase") as f:
            self.assertEqual(f.read(), "Hello world\\\nTest\n")
    
    @evaluate_docstring
    def test_emit_comment(self, rc):
        '''.//Hello world
        .comment No comment
        .emit to file "/tmp/RSLTestCase"'''
        
        with open("/tmp/RSLTestCase") as f:
            self.assertEqual(f.read(), "")
        
    @evaluate_docstring
    def test_emit_flush(self, rc):
        '''Hello world
        .emit to file "/tmp/RSLTestCase"
        .emit to file "/tmp/RSLTestCase"'''
        
        with open("/tmp/RSLTestCase") as f:
            self.assertEqual(f.read(), "")
        
    @evaluate_docstring
    def test_include_after_emit(self, rc):
        '''..exit 1
        .emit to file "/tmp/RSLTestCase"
        .include "/tmp/RSLTestCase"
        .exit 0
        '''
        self.assertEqual(1, rc)
            
    @evaluate_docstring
    def test_emit_after_clear(self, rc):
        '''
        Hello world
        .clear
        .emit to file "/tmp/RSLTestCase"
        '''
        with open("/tmp/RSLTestCase") as f:
            self.assertEqual(f.read(), "")
    
    @evaluate_docstring
    def test_emit_from_function(self, rc):
        '''.function f
Hello world!
        .end function
        .invoke rc = f()
        .emit to file "/tmp/RSLTestCase"
        .exit rc.body
        '''
        
        self.assertEqual("Hello world!\n", rc)
        
        with open("/tmp/RSLTestCase") as f:
            self.assertEqual(f.read(), "")

    @evaluate_docstring
    def test_emit_to_folder(self, rc):
        '''Hello
        .emit to file "/tmp/Some_Test/Folder/RSLTestCase"
        '''
        with open("/tmp/Some_Test/Folder/RSLTestCase") as f:
            self.assertEqual(f.read(), "Hello\n")

        os.remove("/tmp/Some_Test/Folder/RSLTestCase")
        os.rmdir("/tmp/Some_Test/Folder")
        os.rmdir("/tmp/Some_Test")
        
    def test_supress_emit(self):
        self.runtime.emit = 'never'
        code = '''
        Test
        .emit to file "/tmp/RSLTestCase"
        '''
        path = "/tmp/RSLTestCase"
        if os.path.exists(path):
            os.remove(path)
        
        rc = self.eval_text(code, 'test_supress_emit')
        self.assertFalse(rc)
        
        self.assertFalse(os.path.exists(path))
        
    def test_emit_on_change(self):
        self.runtime.emit = 'change'
        code = '''
        Test
        .emit to file "/tmp/RSLTestCase"
        '''
        path = "/tmp/RSLTestCase"
        if os.path.exists(path):
            os.remove(path)
        
        rc = self.eval_text(code, 'test_emit_on_change')
        self.assertFalse(rc)
        self.assertTrue(os.path.exists(path))
        
        t = os.path.getmtime(path)
        time.sleep(0.05)
        self.eval_text('test' + code, 'test_emit_on_change')
        self.assertTrue(t < os.path.getmtime(path))
        
        t = os.path.getmtime(path)
        self.eval_text('test' + code, 'test_emit_on_change')
        self.assertEqual(t, os.path.getmtime(path))
        



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring


class TestBinaryOperation(RSLTestCase):

    def setUp(self):
        RSLTestCase.setUp(self)
        self.metamodel.define_class('A', [('Name', 'string')])
        
    @evaluate_docstring
    def test_plus(self, rc):
        '.exit 1 + 1'
        self.assertEqual(2, rc)
        
    @evaluate_docstring
    def test_minus(self, rc):
        '.exit 1 - 1'
        self.assertEqual(0, rc)

    @evaluate_docstring
    def test_minus_with_unary_minus(self, rc):
        '.exit 1 - -1'
        self.assertEqual(2, rc)
        
    @evaluate_docstring
    def test_multiplication(self, rc):
        '.exit 2 * 2'
        self.assertEqual(4, rc)
        
    @evaluate_docstring
    def test_division(self, rc):
        '.exit 10 / 2'
        self.assertEqual(5, rc)
        
    @evaluate_docstring
    def test_less_true(self, rc):
        '.exit 0 < 1'
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_less_false(self, rc):
        '.exit 0 < 0'
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_less_equal_true(self, rc):
        '.exit 1 <= 1'
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_less_equal_false(self, rc):
        '.exit 2 <= 1'
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_not_equal_false(self, rc):
        '.exit 1 != 1'
        self.assertFalse(rc)
    
    @evaluate_docstring
    def test_great_equal_false(self, rc):
        '.exit 1 >= 2'
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_great_equal_true(self, rc):
        '.exit 3 >= 2'
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_not_equal_true(self, rc):
        '.exit 0 != 1'
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_equal_false(self, rc):
        '.exit 0 == 1'
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_equal_true(self, rc):
        '.exit 1 == 1'
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_grouped(self, rc):
        '''
        .assign x = (1 + 1)
        .exit x
        '''
        self.assertEqual(2, rc)
        
    @evaluate_docstring
    def test_chained(self, rc):
        '''
        .assign x = (1 + 1) + 1
        .exit x
        '''
        self.assertEqual(3, rc)

    @evaluate_docstring
    def test_chained_with_unary(self, rc):
        '''
        .assign x = not (1 == 1)
        .exit x
        '''
        self.assertEqual(False, rc)

    @evaluate_docstring
    def test_and_true(self, rc):
        '''
        .exit True and True
        '''
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_and_false(self, rc):
        '''
        .exit True and False
        '''
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_or_true(self, rc):
        '''
        .exit True or False
        '''
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_or_false(self, rc):
        '''
        .exit False or False
        '''
        self.assertFalse(rc)
        
    @evaluate_docstring
    def test_or_without_spaces(self, rc):
        '''
        .assign x = (True)or(False)
        .exit x
        '''
        self.assertTrue(rc)
        
    @evaluate_docstring
    def test_and_without_spaces(self, rc):
        '''
        .assign x = (True)AND(True)
        .exit x
        '''
        self.assertTrue(rc)

    @evaluate_docstring
    def test_procent(self, rc):
        '''
        .assign x = 5 % 3
        .exit x
        '''
        self.assertEqual(rc, 2)

    @evaluate_docstring
    def test_pipe_with_instances(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .assign a_set = a1 | a2
        .exit cardinality a_set
        '''
        self.assertEqual(2, rc)

    @evaluate_docstring
    def test_pipe_with_instance_and_set(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .create object instance a3 of A
        .select many a_set from instances of A
        
        .assign a_set = a1 | a_set
        .exit cardinality a_set
        '''
        self.assertEqual(3, rc)

    @evaluate_docstring
    def test_caret(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .create object instance a3 of A
        .assign a1.Name = "A1"
        .assign a2.Name = "A2"
        .assign a3.Name = "A3"
        
        .select many a23_set from instances of A where (selected.Name != "A1")
        .select many a13_set from instances of A where (selected.Name != "A2")
        
        .assign a_set = a23_set ^ a13_set
        .exit cardinality a_set .// should not contain a3
        '''
        self.assertEqual(2, rc)

    @evaluate_docstring
    def test_ampesand_with_instances(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .assign a_set = a1 & a2
        .exit cardinality a_set
        '''
        self.assertEqual(0, rc)

    @evaluate_docstring
    def test_ampesand_with_sets(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .create object instance a3 of A
        .assign a1.Name = "A1"
        .assign a2.Name = "A2"
        .assign a3.Name = "A3"

        .select many not_a1_set from instances of A where (selected.Name != "A1")
        .select many not_a2_set from instances of A where (selected.Name != "A2")

        .assign a3_set = not_a1_set & not_a2_set
        .exit cardinality a3_set
        '''
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_instance_plus_instance(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .assign a1.Name = "A1"
        .assign a2.Name = "A2"
        
        .assign a_set = a1 + a2
        .exit cardinality a_set
        '''
        self.assertEqual(2, rc)

    @evaluate_docstring
    def test_instance_minus_instance(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .assign a1.Name = "A1"
        .assign a2.Name = "A2"
        
        .assign a_set = a1 - a2
        .exit cardinality a_set
        '''
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_instance_minus_same_instance(self, rc):
        '''
        .create object instance a1 of A
        .assign a1.Name = "A1"
        
        .assign a_set = a1 - a1
        .exit cardinality a_set
        '''
        self.assertEqual(0, rc)

    @evaluate_docstring
    def test_short_circuit_or(self, rc):
        '''
        .create object instance a1 of A
        .assign a1.Name = "A1"

        .select any a2 from instances of A where ( false )
        .assign cond = ( ( empty a2 ) or ( a2.Name == "a2" ) )
        .exit cond
        '''
        self.assertEqual(True, rc)

    @evaluate_docstring
    def test_short_circuit_and(self, rc):
        '''
        .create object instance a1 of A
        .assign a1.Name = "A1"

        .select any a2 from instances of A where ( false )
        .assign cond = ( ( not_empty a2 ) and ( a2.Name == "a2" ) )
        .exit cond
        '''
        self.assertEqual(False, rc)

    @evaluate_docstring
    def test_instance_set_minus_instance_set(self, rc):
        '''
        .create object instance a1 of A
        .create object instance a2 of A
        .assign a1.Name = "A1"
        .assign a2.Name = "A2"
        .select many an_set from instances of A
        .select many a0_set from instances of A where (false)

        .if ((cardinality (an_set - a0_set)) != 2)
          .exit 1
        .end if

        .if ((cardinality (a0_set - an_set)) != 0)
          .exit 2
        .end if

        .if ((cardinality (an_set - a1)) != 1)
          .exit 3
        .end if

        .if ((cardinality (a1 - an_set)) != 0)
          .exit 4
        .end if

        .if ((cardinality (a0_set - a1)) != 0)
          .exit 5
        .end if

        .if ((cardinality (a1 - a0_set)) != 1)
          .exit 6
        .end if

        .exit 0
        '''
        self.assertEqual(0, rc)




# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring


class TestLoop(RSLTestCase):

    @evaluate_docstring
    def test_while_loop(self, rc):
        '''
        .assign x = 10
        .while (x > 0)
            .assign x = x - 1
        .end while
        .exit x
        '''
        self.assertEqual(0, rc)

    @evaluate_docstring
    def test_while_loop_with_comments(self, rc):
        '''
        .assign x = 10
        .while (x > 0) .//
            .if (x == 5)
                .break while .//
            .end if
            .assign x = x - 1 .//
        .end while .//
        .exit x
        '''
        self.assertEqual(5, rc)
        
    @evaluate_docstring
    def test_while_loop_break(self, rc):
        '''
        .assign x = 10
        .while (x > 0)
            .if (x == 5)
                .break while
            .end if
            .assign x = x - 1
        .end while
        .exit x
        '''
        self.assertEqual(5, rc)
        
    def test_for_loop(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign x = 0
        .for each a in a_set
            .assign x = x + 1
        .end for
        .exit x
        '''
        
        for i in range(0, 10):
            rc = self.eval_text(text)
            self.assertEqual(i, rc)
            self.metamodel.new('A')

    def test_for_loop_with_comments(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign x = 0
        .for each a in a_set .//
            .if (x == 3) .//
                .break for .//
            .end if .//
            .assign x = x + 1 .//
        .end for .//
        .exit x
        '''

        for _ in range(0, 10):
            self.metamodel.new('A')
            
        rc = self.eval_text(text)
        self.assertEqual(3, rc)

    def test_for_loop_break(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign x = 0
        .for each a in a_set
            .if (x == 3)
                .break for
            .end if
            .assign x = x + 1
        .end for
        .exit x
        '''

        for _ in range(0, 10):
            self.metamodel.new('A')
            
        rc = self.eval_text(text)
        self.assertEqual(3, rc)

    def test_first_in_loop_count(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign x = 0
        .for each a in a_set
            .if (first a_set)
                .assign x = x + 1
            .end if
        .end for
        .exit x
        '''
        
        for _ in range(0, 10):
            self.metamodel.new('A')
            rc = self.eval_text(text)
            self.assertEqual(1, rc)

    def test_not_first_in_loop_count(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign x = 0
        .for each a in a_set
            .if (not_first a_set)
                .assign x = x + 1
            .end if
        .end for
        .exit x
        '''
        
        for i in range(0, 10):
            self.metamodel.new('A')
            rc = self.eval_text(text)
            self.assertEqual(i, rc)
        
    def test_last_in_loop_count(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign x = 0
        .for each a in a_set
            .if (last a_set)
                .assign x = x + 1
            .end if
        .end for
        .exit x
        '''
        
        for _ in range(0, 10):
            self.metamodel.new('A')
            rc = self.eval_text(text)
            self.assertEqual(1, rc)
        
    def test_not_last_in_loop_count(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign x = 0
        .for each a in a_set
            .if (not_last a_set)
                .assign x = x + 1
            .end if
        .end for
        .exit x
        '''
        
        for i in range(0, 10):
            self.metamodel.new('A')
            rc = self.eval_text(text)
            self.assertEqual(i, rc)
    
    def test_first_in_loop(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign loop_count = 0
        .for each a in a_set
            .if (first a_set)
                .exit loop_count
            .end if
            .assign loop_count = (loop_count + 1)
        .end for
        .exit -1
        '''
        
        for _ in range(0, 10):
            self.metamodel.new('A')
            rc = self.eval_text(text)
            self.assertEqual(0, rc)
        
    def test_not_first_is_second_in_loop(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign loop_count = 0
        .for each a in a_set
            .if (not_first a_set)
                .exit loop_count
            .end if
            .assign loop_count = (loop_count + 1)
        .end for
        .exit loop_count
        '''
        
        for _ in range(0, 10):
            self.metamodel.new('A')
            rc = self.eval_text(text)
            self.assertEqual(1, rc)
        
    def test_last_in_loop(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign loop_count = 0
        .for each a in a_set
            .if (last a_set)
                .exit loop_count
            .end if
            .assign loop_count = (loop_count + 1)
        .end for
        .exit -1
        '''
        
        for i in range(0, 10):
            self.metamodel.new('A')
            rc = self.eval_text(text)
            self.assertEqual(i, rc)

    def test_first_element_is_not_last_in_loop(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a_set from instances of A
        .assign loop_count = 0
        .for each a in a_set
            .if (not_last a_set)
                .exit loop_count
            .end if
            .assign loop_count = (loop_count + 1)
        .end for
        .exit -1
        '''
        
        for _ in range(0, 5):
            self.metamodel.new('A')
        
        rc = self.eval_text(text)
        self.assertEqual(0, rc)
            
    def test_shadowing_selected(self):
        self.metamodel.define_class('A', [('ID', 'integer')])
        self.metamodel.define_class('B', [('ID', 'integer')])
        
        text = '''
        .select many a_set from instances of A
        .for each a in a_set
          .select many b_set from instances of B where (selected.ID > 5)
          .assign x = 0
          .assign y = 0
          .for each b in b_set
            .assign x = (x + (first b_set))
            .assign y = (y + (first a_set))
          .end for
          .exit "${x} ${y}"
        .end for
        .exit 1
        '''

        self.metamodel.new('A', ID=1);
        self.metamodel.new('A', ID=2);
        self.metamodel.new('A', ID=3);
        
        self.metamodel.new('B', ID=6);
        self.metamodel.new('B', ID=7);
        self.metamodel.new('B', ID=8);

        rc = self.eval_text(text)
        self.assertEqual("1 3", rc)
        



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

import xtuml

from utils import RSLTestCase

from rsl.parse import ParseException
from rsl.runtime import RuntimeException


class TestSelect(RSLTestCase):

    def test_select_any_empty(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select any a from instances of A
        .exit empty a
        '''
        
        rc = self.eval_text(text)
        self.assertTrue(rc)
        
        self.metamodel.new('A')
        
        rc = self.eval_text(text)
        self.assertFalse(rc)

    def test_select_any_not_empty(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select any a from instances of A
        .exit not_empty a
        '''
        
        rc = self.eval_text(text)
        self.assertFalse(rc)
        
        self.metamodel.new('A')
        
        rc = self.eval_text(text)
        self.assertTrue(rc)
        
    def test_select_many_empty(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select many a from instances of A
        .exit empty a
        '''
        
        rc = self.eval_text(text)
        self.assertTrue(rc)
        
        self.metamodel.new('A')
        
        rc = self.eval_text(text)
        self.assertFalse(rc)
        
    def test_select_many_not_empty(self):
        self.metamodel.define_class('A', [])
        
        text = '''
        .select many a from instances of A
        .exit not_empty a
        '''
        
        rc = self.eval_text(text)
        self.assertFalse(rc)
        
        self.metamodel.new('A')
                
        rc = self.eval_text(text)
        self.assertTrue(rc)
        
    def test_select_many_cardinality(self):
        self.metamodel.define_class('A', [])
        
        text = '''
        .select many a from instances of A
        .exit cardinality a
        '''
        
        for i in range(0, 10):
            rc = self.eval_text(text)
            self.assertEqual(i, rc)
            self.metamodel.new('A')

    def test_select_one_from_instances(self):
        self.assertRaises(ParseException, self.eval_text,
                          '.select one a from instances of A')
        
    def test_select_many_ordered_by(self):
        self.metamodel.define_class('A', [('color', 'string'), ('num', 'integer')])
        self.metamodel.new('A', color='red',    num=0)
        self.metamodel.new('A', color='orange', num=1)
        self.metamodel.new('A', color='yellow', num=0)
        self.metamodel.new('A', color='green',  num=1)
        self.metamodel.new('A', color='blue',   num=0)
        self.metamodel.new('A', color='purple', num=1)

        text = '''
        .select many as from instances of A ordered_by (num, color)
        .assign colors = ""
        .for each a in as
          .assign colors = "${colors} ${a.color}"
        .end for
        .exit colors
        '''

        rc = self.eval_text(text)
        self.assertEqual(rc, ' blue red yellow green orange purple')

    def test_select_many_reverse_ordered_by(self):
        self.metamodel.define_class('A', [('color', 'string'), ('num', 'integer')])
        self.metamodel.new('A', color='red',    num=0)
        self.metamodel.new('A', color='orange', num=1)
        self.metamodel.new('A', color='yellow', num=0)
        self.metamodel.new('A', color='green',  num=1)
        self.metamodel.new('A', color='blue',   num=0)
        self.metamodel.new('A', color='purple', num=1)

        text = '''
        .select many as from instances of A reverse_ordered_by (num, color)
        .assign colors = ""
        .for each a in as
          .assign colors = "${colors} ${a.color}"
        .end for
        .exit colors
        '''

        rc = self.eval_text(text)
        self.assertEqual(rc, ' purple orange green yellow red blue')

    def test_select_when_created(self):
        self.metamodel.define_class('A', [])

        text = '''
        .create object instance a of A
        .select any b from instances of A
        .exit empty b
        '''
        rc = self.eval_text(text)
        self.assertFalse(rc)

    def test_select_one_navigation(self):
        self.metamodel.define_class('A', [('Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id'), ('A_Id', 'unique_id')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='A',
                                          target_kind='B',
                                          source_keys=['Id'],
                                          target_keys=['A_Id'],
                                          source_many=False,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=True,
                                          source_phrase='',
                                          target_phrase='')
        
        a = self.metamodel.new('A')
        b = self.metamodel.new('B')
        xtuml.relate(a, b, 1)

        text = '''
        .select any a from instances of A
        .select one b related by a->B[R1]
        .exit b.Id
        '''
        rc = self.eval_text(text)
        self.assertEqual(b.Id, rc)

    def test_select_any_navigation(self):
        '''
        |===================|                |======================|  
        |         A         |                |         B            |
        |-------------------| 1           *  |----------------------|
        | Id: unique_id {I} | -------------- | Id: unique_id    {I} |
        |===================|       R1       | A_Id: unique_id {R1} |
                                             |======================|
        '''
        self.metamodel.define_class('A', [('Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id'), 
                                          ('A_Id', 'unique_id')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='B',
                                          target_kind='A',
                                          source_keys=['A_Id'],
                                          target_keys=['Id'],
                                          source_many=True,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=False,
                                          source_phrase='',
                                          target_phrase='')
        
        a = self.metamodel.new('A')
        b = self.metamodel.new('B')
        xtuml.relate(a, b, 1)
        
        text = '''
        .select any a from instances of A
        .select any b related by a->B[R1]
        .exit b.Id
        '''
        
        rc = self.eval_text(text)
        self.assertEqual(b.Id, rc)

    def test_select_many_navigation(self):
        '''
        |===================|                |======================|  
        |         A         |                |         B            |
        |-------------------| 1           *  |----------------------|
        | Id: unique_id {I} | -------------- | Id: unique_id    {I} |
        |===================|       R1       | A_Id: unique_id {R1} |
                                             |======================|
        '''
        self.metamodel.define_class('A', [('Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id'), 
                                          ('A_Id', 'unique_id')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='B',
                                          target_kind='A',
                                          source_keys=['A_Id'],
                                          target_keys=['Id'],
                                          source_many=True,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=False,
                                          source_phrase='',
                                          target_phrase='')
        
        a = self.metamodel.new('A')
        xtuml.relate(a, self.metamodel.new('B'), 1)
        xtuml.relate(a, self.metamodel.new('B'), 1)
        xtuml.relate(a, self.metamodel.new('B'), 1)

        text = '''
        .select any a from instances of A
        .select many bs related by a->B[R1]
        .exit cardinality bs
        '''
        
        rc = self.eval_text(text)
        self.assertEqual(3, rc)


    def test_select_one_from_many_navigation(self):
        '''
        |===================|                |======================|
        |         A         |                |         B            |
        |-------------------| 1           *  |----------------------|
        | Id: unique_id {I} | -------------- | Id: unique_id    {I} |
        |===================|       R1       | A_Id: unique_id {R1} |
                                             |======================|
        '''
        self.metamodel.define_class('A', [('Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id'),
                                          ('A_Id', 'unique_id')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='B',
                                          target_kind='A',
                                          source_keys=['A_Id'],
                                          target_keys=['Id'],
                                          source_many=True,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=False,
                                          source_phrase='',
                                          target_phrase='')
        
        a = self.metamodel.new('A')
        xtuml.relate(a, self.metamodel.new('B'), 1)
        xtuml.relate(a, self.metamodel.new('B'), 1)
        xtuml.relate(a, self.metamodel.new('B'), 1)

        text = '''
        .select any a from instances of A
        .select one b related by a->B[R1]
        '''

        rc = self.eval_text(text)
        self.assertIsInstance(rc, RuntimeException)
        
    def test_select_many_navigation_ordered_by(self):
        '''
        |===================|                |======================|
        |         A         |                |         B            |
        |-------------------| 1           *  |----------------------|
        | Id: unique_id {I} | -------------- | Id: unique_id    {I} |
        |===================|       R1       | A_Id: unique_id {R1} |
                                             | color: string        |
                                             | num: integer         |
                                             |======================|
        '''
        self.metamodel.define_class('A', [('Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id'), 
                                          ('A_Id', 'unique_id'),
                                          ('color', 'string'),
                                          ('num', 'integer')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='B',
                                          target_kind='A',
                                          source_keys=['A_Id'],
                                          target_keys=['Id'],
                                          source_many=True,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=False,
                                          source_phrase='',
                                          target_phrase='')

        a = self.metamodel.new('A')
        xtuml.relate(a, self.metamodel.new('B', color='red',    num=0), 1)
        xtuml.relate(a, self.metamodel.new('B', color='orange', num=1), 1)
        xtuml.relate(a, self.metamodel.new('B', color='yellow', num=0), 1)
        xtuml.relate(a, self.metamodel.new('B', color='green',  num=1), 1)
        xtuml.relate(a, self.metamodel.new('B', color='blue',   num=0), 1)
        xtuml.relate(a, self.metamodel.new('B', color='purple', num=1), 1)

        text = '''
        .select any a from instances of A
        .select many bs related by a->B[R1] ordered_by (num, color)
        .assign colors = ""
        .for each b in bs
          .assign colors = "${colors} ${b.color}"
        .end for
        .exit colors
        '''
        rc = self.eval_text(text)
        self.assertEqual(rc, ' blue red yellow green orange purple')
     
    def test_select_one_reflexive_navigation(self):
        self.metamodel.define_class('A', [('Id', 'unique_id'),
                                          ('Next_Id', 'unique_id'),
                                          ('Name', 'string')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='A',
                                          target_kind='A',
                                          source_keys=['Id'],
                                          target_keys=['Next_Id'],
                                          source_many=False,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=True,
                                          source_phrase='prev',
                                          target_phrase='next')

        first = self.metamodel.new('A', Name="First")
        second = self.metamodel.new('A', Name="Second")
        xtuml.relate(first, second, 1, 'prev')
        
        text = '''
        .select any first_inst from instances of A where (selected.Name == "First")
        .select one second_inst related by first_inst->A[R1.'prev']
        .exit second_inst.Name
        '''
        rc = self.eval_text(text)
        self.assertEqual(second.Name, rc)

        text = '''
        .select any second_inst from instances of A where (selected.Name == "Second")
        .select one first_inst related by second_inst->A[R1.'next']
        .exit first_inst.Name
        '''
        rc = self.eval_text(text)
        self.assertEqual(first.Name, rc)
        
    def test_select_any_substitution_navigation(self):
        self.metamodel.define_class('A', [('Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id'), 
                                          ('A_Id', 'unique_id'), 
                                          ('Name', 'string')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='A',
                                          target_kind='B',
                                          source_keys=[],
                                          target_keys=[],
                                          source_many=False,
                                          target_many=True,
                                          source_conditional=True,
                                          target_conditional=True,
                                          source_phrase='',
                                          target_phrase='')

        a = self.metamodel.new('A')
        b = self.metamodel.new('B', Name='Test')
        xtuml.relate(a, b, 1)
        
        text = '''
        .select any a from instances of A where ("${selected->B[R1].name}" == "Test")
        .exit a.Id
        '''
        
        rc = self.eval_text(text)
        self.assertEqual(a.Id, rc)
        
    def test_select_with_many_spaces(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select       any        a       from          instances       of      A
        .select       many       a_set   from          instances       of      A
        '''
        
        self.eval_text(text)

    def test_select_with_type_name(self):
        self.metamodel.define_class('A', [])

        text = '''
        .select any string from instances of A
        '''
        
        self.eval_text(text)
        



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring


class TestConstants(RSLTestCase):

    @evaluate_docstring
    def test_positive_integer(self, rc):
        '.exit 1'
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_negative_integer(self, rc):
        '.exit -1'
        self.assertEqual(-1, rc)

    @evaluate_docstring
    def test_positive_real(self, rc):
        '''
        .exit 1.1
        '''
        self.assertEqual(1.1, rc)
        
    @evaluate_docstring
    def test_negative_real(self, rc):
        '''
        .exit -1.1
        '''
        self.assertEqual(-1.1, rc)

    @evaluate_docstring
    def test_true(self, rc):
        '''
        .exit true
        '''
        self.assertEqual(True, rc)
        
    @evaluate_docstring
    def test_false(self, rc):
        '''
        .exit false
        '''
        self.assertEqual(False, rc)
        
    @evaluate_docstring
    def test_string(self, rc):
        '.exit "Hello"'
        self.assertEqual("Hello", rc)


    @evaluate_docstring
    def test_empty_string(self, rc):
        '''
        .exit ""
        '''
        self.assertEqual("", rc)
        



# encoding: utf-8
# Copyright (C) 2015 Per Jonsson

from utils import RSLTestCase
from utils import evaluate_docstring


class TestInclude(RSLTestCase):

    @evaluate_docstring
    def test_reinclude(self, rc):
        '''
        ..assign attr_x = 1
        .emit to file "/tmp/RSLTestCase"
        .function f
          .include "/tmp/RSLTestCase"
        .end function
        .invoke res = f()
        .clear
        .assign attr_x = 2
        .emit to file "/tmp/RSLTestCase"
        .function g
          .include "/tmp/RSLTestCase"
        .end function
        .invoke res2 = g()
        .if (res.x != res2.x)
          .exit 1
        .end if
        .exit 0
        '''
        self.assertEqual(0, rc)

    @evaluate_docstring
    def test_relative_include(self, rc):
        '''
        .include "tests/test_files/foo.rsl"
        '''
        self.assertEqual(4711, rc)

    @evaluate_docstring
    def test_invalid_file(self, rc):
        '''
        .include "tests/test_files/nonexisting_file"
        '''
        self.assertIsInstance(rc, Exception)




# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring


class TestParseKeyword(RSLTestCase):

    @evaluate_docstring
    def test_simple_keyword(self, rc):
        '''
        .assign s = "TEST:Hello world!"
        .exit "${s:TEST}"
        '''
        self.assertEqual("Hello world!", rc)

    @evaluate_docstring
    def test_keyword_as_variable(self, rc):
        '''
        .assign kw = "KEYWORD"
        .assign s = "KEYWORD:Hello!"
        .exit "${s:${kw}}"
        '''
        self.assertEqual("Hello!", rc)

    @evaluate_docstring
    def test_keyword_missmatch(self, rc):
        '''
        .assign s = "TEST:Hello world!"
        .exit "${s:test}"
        '''
        self.assertEqual("", rc)
        
    @evaluate_docstring
    def test_keyword_with_spaces(self, rc):
        '''
        .assign kw = "KEYWORD"
        .assign s = "KEYWORD: Hello!   "
        .exit "${s:${kw}}"
        '''
        self.assertEqual("Hello!", rc)
        


# encoding: utf-8
# Copyright (C) 2015 John Törnblom

import unittest
import tempfile
import sys
import os
import stat
import logging
import pytest

from io import StringIO
    
import rsl


class TestCommandLineInterface(unittest.TestCase):
    
    def setUp(self):
        self.temp_files = set()
        logger = logging.getLogger()
        self.log = logging.StreamHandler(StringIO())
        logger.addHandler(self.log)
        
    def tearDown(self):
        logger = logging.getLogger()
        logger.removeHandler(self.log)
        
        for temp in self.temp_files:
            temp.close()
            os.remove(temp.name)

    def temp_file(self, mode='w'):
        temp = tempfile.NamedTemporaryFile(mode=mode, delete=False)
        self.temp_files.add(temp)
        return temp
    
    def test_unsed_arguments(self):
        argv = ['test_unsed_arguments', 
                '-nopersist', #don't save database to disk during testing
                '-d', '1',
                '-priority', '32',
                '-lVHs', '-lSCs', '-l2b', '-l2s', '-l3b', '-l3s',
                '-e', 'some_string',
                '-t', 'some_string',
                '-v', 'STMT',
                '-q',
                '-l',
                '-#', '4']
        rsl.main(argv)

    def test_ignoring_arguments(self):
        argv = ['test_ignoring_arguments', 
                '-nopersist', #don't save database to disk during testing
                '-ignore_rest', 'some', 'more']
        rsl.main(argv)

    def test_invalid_arguments(self):
        argv = ['test_invalid_arguments', 
                'some', 'more']
        self.assertRaises(SystemExit, rsl.main, argv)
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertIn('ERROR', output)
    
    def test_help(self):
        argv = ['test_help', 
                '-h']
        
        self.assertRaises(SystemExit, rsl.main, argv)
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertIn('USAGE', output)
    
    def test_version(self):
        argv = ['test_version', 
                '-version']
        
        self.assertRaises(SystemExit, rsl.main, argv)
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertIn(rsl.version.complete_string, output)
        
    def test_integrity(self):
        schema = self.temp_file(mode='w')
        script = self.temp_file(mode='w')
        
        schema.file.write('CREATE TABLE Cls (Id STRING);')
        schema.file.write('CREATE UNIQUE INDEX I1 ON Cls (Id);')
        schema.file.flush()
        
        script.file.write('.create object instance cls of Cls\n')
        script.file.write('.assign cls.Id = "test"\n')
        
        argv = ['test_integrity', 
                '-nopersist',
                '-integrity',
                '-import', schema.name,
                '-arch', script.name]
        
        self.assertEqual(0, rsl.main(argv))
    
        # expect two uniqueness constrain violations
        script.file.write('.create object instance cls of Cls\n')
        script.file.write('.assign cls.Id = "test"\n')
        script.file.flush()
        self.assertEqual(1, rsl.main(argv))
    
    def test_qim(self):
        schema = self.temp_file(mode='w')
        modeldata = self.temp_file(mode='w')
        script = self.temp_file(mode='w')
        
        schema.file.write('CREATE TABLE Cls (Id STRING, OtherAttr INTEGER);')
        schema.file.write('CREATE UNIQUE INDEX I1 ON Cls (Id);')
        schema.file.flush()
        
        modeldata.file.write('INSERT INTO Cls VALUES ("myid");')
        modeldata.file.flush()
        
        script.file.write('')
        script.file.flush()
        
        argv = ['test_qim', 
                '-nopersist',
                '-import', schema.name,
                '-import', modeldata.name,
                '-arch', script.name]
        rsl.main(argv)
        
        argv = ['test_qim', 
                '-nopersist',
                '-qim',
                '-import', schema.name,
                '-import', modeldata.name,
                '-arch', script.name]
        rsl.main(argv)

        mismatches = self.log.stream.getvalue().count('mismatch')
        self.assertEqual(1, mismatches)
        
    def test_include(self):
        script = self.temp_file(mode='w')
        script.file.write('.print "Hello"\n')
        script.file.write('.include "spam.inc"\n')
        script.file.flush()
        
        argv = ['test_include', 
                '-arch', script.name,
                '-include', 
                os.path.dirname(__file__) + os.path.sep + 'test_files',
                '-nopersist']
        
        rsl.main(argv)
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertIn('Hello', output)

    def test_nopersist(self):
        db_filename = tempfile.mktemp()
        script = self.temp_file(mode='w')

        script.file.write('.print "Hello"\n')
        script.file.flush()
        
        argv = ['test_nopersist', 
                '-f', db_filename,
                '-arch', script.name,
                '-nopersist']
        
        rsl.main(argv)
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertIn('Hello', output)
        self.assertFalse(os.path.exists(db_filename))
    
    def test_persist(self):
        db = self.temp_file(mode='r')
        schema = self.temp_file(mode='w')
        script = self.temp_file(mode='w')
        
        schema.file.write('CREATE TABLE Cls (Id UNIQUE_ID);')
        schema.file.flush()
        
        script.file.write('.create object instance cls of Cls\n')
        script.file.write('.print "Hello"\n')
        script.file.flush()
        
        argv = ['test_persist', 
                '-f', db.name,
                '-import', schema.name,
                '-arch', script.name]
        
        rsl.main(argv)
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertIn('Hello', output)
        
        with open(db.name, 'r') as f:
            s = f.read()
            self.assertIn('CREATE TABLE', s)
            self.assertIn('INSERT INTO', s)
    
    def test_disable_emit(self):
        emit_filename = tempfile.mktemp().replace('\\', '/')
        script = self.temp_file(mode='w')
        script.file.write('Test\n')
        script.file.write('.emit to file "%s"\n' % emit_filename)
        script.file.flush()
        
        argv = ['test_disable_emit', 
                '-arch', script.name,
                '-emit', 'never',
                '-nopersist']
        
        rsl.main(argv)
        self.assertFalse(os.path.exists(emit_filename))

    def test_diff(self):
        diff = self.temp_file(mode='r')
        emit = self.temp_file(mode='r')
        script = self.temp_file(mode='w')
        script.file.write('Hello file\n')
        script.file.write('.emit to file "%s"\n' % emit.name.replace('\\', '/'))
        script.file.flush()
        
        argv = ['test_diff', 
                '-arch', script.name,
                '-diff', diff.name,
                '-nopersist']
        
        rsl.main(argv)
        with open(diff.name, 'r') as f:
            s = f.read()
            self.assertIn('test_diff -arch', s)
            self.assertIn('Hello file', s)
        
    def test_force(self):
        emit = self.temp_file(mode='r')
        script = self.temp_file(mode='w')
        script.file.write('Hello file\n')
        script.file.write('.emit to file "%s"\n' % emit.name.replace('\\', '/'))
        script.file.flush()
        
        os.chmod(emit.name, stat.S_IRUSR)

        argv = ['test_force', 
                '-arch', script.name,
                '-nopersist']

        self.assertRaises(SystemExit, rsl.main, argv)
        argv.append('-force')
        rsl.main(argv)
        
        with open(emit.name, 'r') as f:
            s = f.read()
            self.assertIn('Hello file', s)

    def test_dumpsql(self):
        dump = self.temp_file(mode='r')
        schema = self.temp_file(mode='w')
        schema.file.write('CREATE TABLE Person (Name STRING, Age INTEGER);\n')
        schema.file.flush()

        script = self.temp_file(mode='w')
        script.file.write('.create object instance person1 of Person\n')
        script.file.write('.assign person1.Name = "Levi"\n')
        script.file.write('.assign person1.Age = 24\n')
        script.file.flush()

        argv = ['test_dumpsql',
                '-nopersist',
                '-dumpsql', dump.name,
                '-import', schema.name,
                '-arch', script.name]

        self.assertEqual(0, rsl.main(argv))

        with open(dump.name, 'r') as f:
            s = f.read()
            self.assertIn('INSERT INTO Person', s)

    @pytest.fixture(autouse=True)
    def capfd(self, capfd):
        self.capfd = capfd


if __name__ == "__main__":
    unittest.main()




# encoding: utf-8
# Copyright (C) 2015 John Törnblom

import datetime
import getpass
import os

from utils import RSLTestCase
from utils import evaluate_docstring

import rsl.version


class TestInfo(RSLTestCase):

    @evaluate_docstring
    def test_date(self, rc):
        '.exit "${info.date}"'
        now = datetime.datetime.now()
        now = datetime.datetime.ctime(now)
        now = str(now)
        self.assertEqual(now, rc)

    @evaluate_docstring
    def test_user_id(self, rc):
        '.exit "${info.user_id}"'
        self.assertEqual(getpass.getuser(), rc)

    @evaluate_docstring
    def test_uuid(self, rc):
        '.exit "${info.unique_num}"'
        self.assertEqual("1", rc)
        
    @evaluate_docstring
    def test_arch_file_name(self, rc):
        '.exit "${info.arch_file_name}"'
        self.assertEqual("test_info.test_arch_file_name", rc)
    
    def test_arch_file_name_with_folder(self):
        text = '.exit "${info.arch_file_name}"'
        rc = self.eval_text(text, '/dev/stdin')
        self.assertEqual("stdin", rc)
    
    def test_arch_file_path(self):
        text = '.exit "${info.arch_file_path}"'
        rc = self.eval_text(text, '/dev/stdin')
        self.assertEqual("/dev/stdin", rc)
        
    def test_arch_folder_path(self):
        text = '.exit "${info.arch_folder_path}"'
        rc = self.eval_text(text, '/dev/stdin')
        self.assertEqual("/dev", rc)
    
    @evaluate_docstring
    def test_arch_file_line(self, rc):
        '''
        .// No comment
        .exit "${info.arch_file_line}"
        '''
        self.assertEqual("3", rc)

    @evaluate_docstring
    def test_version(self, rc):
        '.exit "${info.interpreter_version}"'
        self.assertEqual(rsl.version.complete_string, rc)

    @evaluate_docstring
    def test_platform(self, rc):
        '.exit "${info.interpreter_platform}"'
        self.assertEqual(os.name, rc)
        





# encoding: utf-8
# Copyright (C) 2018 John Törnblom
'''
Example on how to extend the functionality of rsl.gen_erate by defining a custom
string formatter that invoke clang-format.

Bundle into gen_erate.pyz using the --main flag to setup.py, e.g.
    python setup.py bundle --main=examples/clangformat.py
'''
import sys
import subprocess

from rsl import gen_erate
from rsl import string_formatter


@string_formatter('t')
def format_cpp(s):
    proc = subprocess.Popen('clang-format',
                            stdout=subprocess.PIPE,
                            stdin=subprocess.PIPE)
    proc.stdin.write(s.encode('utf-8'))
    proc.stdin.close()
    res = proc.stdout.read()
    proc.wait()

    if proc.returncode:
        return s
    else:
        return res.decode('utf-8')


def main():
    print('Running my custom version of gen_erate')
    rc = gen_erate.main()
    sys.exit(rc)


if __name__ == '__main__':
    main()



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring

from rsl.runtime import RuntimeException


class TestFormat(RSLTestCase):

    @evaluate_docstring
    def test_uppercase(self, rc):
        '''
        .assign x = "hello"
        .exit "$U{x}"
        '''
        self.assertEqual("HELLO", rc)

    @evaluate_docstring
    def test_lowercase(self, rc):
        '''
        .assign x = "HELlo"
        .exit "$L{x}"
        '''
        self.assertEqual("hello", rc)

    @evaluate_docstring
    def test_capitalize(self, rc):
        '''
        .assign x = "heLLO"
        .exit "$C{x}"
        '''
        self.assertEqual("Hello", rc)

    @evaluate_docstring
    def test_cr_euqlas_rc(self, rc):
        '''
        .// The removal of whitespace occurs after the capitalization
        .// has taken place in the case of the CR or RC combination.
        .assign x = "hello world"
        .exit "$cr{x} $rc{x}"
        '''
        self.assertEqual("HelloWorld HelloWorld", rc)

    @evaluate_docstring
    def test_capitalize_with_underscore(self, rc):
        '''
        .assign x = "test_something_else"
        .exit "$C{x}"
        '''
        self.assertEqual("Test_something_else", rc)
    
    @evaluate_docstring
    def test_remove_splat(self, rc):
        '''
        .assign x = "*splat*splat*"
        .exit "$tnosplat{x}"
        '''
        self.assertEqual("splatsplat", rc)
        
    @evaluate_docstring
    def test_escape_single_quot(self, rc):
        '''
        .assign x = "'"
        .exit "$t2tick{x}"
        '''
        self.assertEqual("''", rc)
        
    @evaluate_docstring
    def test_underscore(self, rc):
        '''
        .assign x = "hello world"
        .exit "$_{x}"
        '''
        self.assertEqual("hello_world", rc)

    @evaluate_docstring
    def test_corba(self, rc):
        '''
        .assign x = "HelLO woRLd"
        .exit "$o{x}"
        '''
        self.assertEqual("helloWorld", rc)

    @evaluate_docstring
    def test_corba_with_underline(self, rc):
        '''
        .assign x = "Hello_world!"
        .exit "$o{x}"
        '''
        self.assertEqual("helloWorld", rc)
        
    @evaluate_docstring
    def test_corba_with_empty_string(self, rc):
        '''
        .assign x = ""
        .exit "$o{x}"
        '''
        self.assertEqual("", rc)
        
    @evaluate_docstring
    def test_example_uppercase(self, rc):
        '''
        .assign x = "Example Text"
        .exit "$u{x}"
        '''
        self.assertEqual("EXAMPLE TEXT", rc)
        
    @evaluate_docstring
    def test_uppercase_with_underscore(self, rc):
        '''
        .assign x = "Example Text"
        .exit "$u_{x}"
        '''
        self.assertEqual("EXAMPLE_TEXT", rc)

    @evaluate_docstring
    def test_uppercase_with_remove_spaces(self, rc):
        '''
        .assign x = "Example Text"
        .exit "$ur{x}"
        '''
        self.assertEqual("EXAMPLETEXT", rc)

    @evaluate_docstring
    def test_example_c(self, rc):
        '''
        .assign x = "ExamplE TExt"
        .exit "$c{x}"
        '''
        self.assertEqual("Example Text", rc)
        
    @evaluate_docstring
    def test_example_c_(self, rc):
        '''
        .assign x = "ExamplE TExt"
        .exit "$c_{x}"
        '''
        self.assertEqual("Example_Text", rc)
        
    @evaluate_docstring
    def test_example_cr(self, rc):
        '''
        .assign x = "ExamplE TExt"
        .exit "$cr{x}"
        '''
        self.assertEqual("ExampleText", rc)
        
    @evaluate_docstring
    def test_underscore_with_lowercase(self, rc):
        '''
        .assign x = "ExamplE TExt"
        .exit "$l{x}"
        '''
        self.assertEqual("example text", rc)
        
    @evaluate_docstring
    def test_lowercase_with_underscore(self, rc):
        '''
        .assign x = "ExamplE TExt"
        .exit "$l_{x}"
        '''
        self.assertEqual("example_text", rc)
        
    @evaluate_docstring
    def test_example_lr(self, rc):
        '''
        .assign x = "ExamplE TExt"
        .exit "$lr{x}"
        '''
        self.assertEqual("exampletext", rc)
        
    @evaluate_docstring
    def test_example_o(self, rc):
        '''
        .assign x = "ExamplE@34 TExt"
        .exit "$o{x}"
        '''
        self.assertEqual("example34Text", rc)

    @evaluate_docstring
    def test_no_linebreak(self, rc):
        '''
        .assign x = "foo\nbar"
        .exit "$tnonl{x}"
        '''
        self.assertEqual("foo bar", rc)
        
    @evaluate_docstring
    def test_chain(self, rc):
        '''
        .function assertEquals
          .param string actual
          .param string expected
          .param integer test_num
          .if ( actual != expected )
            .exit "chain test ${test_num} failed, expected = ${expected} actual = ${actual}"
          .end if
        .end function
        
        .function tcf_kl_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcf_kl{test_string}", expected, test_num)
        .end function
        
        .function tcb_kl_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcb_kl{test_string}", expected, test_num)
        .end function
        
        .function tcf_rel_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcf_rel{test_string}", expected, test_num)
        .end function
        
        .function tcb_rel_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcb_rel{test_string}", expected, test_num)
        .end function
        
        .function tcf_phrase_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcf_phrase{test_string}", expected, test_num)
        .end function
        
        .function tcb_phrase_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcb_phrase{test_string}", expected, test_num)
        .end function
        
        .function tcf_rest_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcf_rest{test_string}", expected, test_num)
        .end function
        
        .function tcb_rest_test
          .param string test_string
          .param string expected
          .param integer test_num
          .invoke assertEquals("$tcb_rest{test_string}", expected, test_num)
        .end function

        .assign test_string0 = "->A[R1]->B[R2]"
        .assign test_string1 = "->C[R3]->D[R4.'b1 phrase']"
        .assign test_string2 = "->E[R5.'a2 phrase']->F[R6]"
        .assign test_string3 = "->G[R7.'a3 phrase']->H[R8.'b3 phrase']"
        .assign test_string4 = ""
        
        .invoke tcf_kl_test(test_string0, "A", 1)
        .invoke tcf_kl_test(test_string1, "C", 2)
        .invoke tcf_kl_test(test_string2, "E", 3)
        .invoke tcf_kl_test(test_string3, "G", 4)
        .invoke tcb_kl_test(test_string0, "B", 5)
        .invoke tcb_kl_test(test_string1, "D", 6)
        .invoke tcb_kl_test(test_string2, "F", 7)
        .invoke tcb_kl_test(test_string3, "H", 8)
        
        .invoke tcf_rel_test(test_string0, "1", 9)
        .invoke tcf_rel_test(test_string1, "3", 10)
        .invoke tcf_rel_test(test_string2, "5", 11)
        .invoke tcf_rel_test(test_string3, "7", 12)
        .invoke tcb_rel_test(test_string0, "2", 13)
        .invoke tcb_rel_test(test_string1, "4", 14)
        .invoke tcb_rel_test(test_string2, "6", 15)
        .invoke tcb_rel_test(test_string3, "8", 16)
        
        .invoke tcf_rest_test(test_string0, "->B[R2]", 17)
        .invoke tcf_rest_test(test_string1, "->D[R4.'b1 phrase']", 18)
        .invoke tcf_rest_test(test_string2, "->F[R6]", 19)
        .invoke tcf_rest_test(test_string3, "->H[R8.'b3 phrase']", 20)
        .invoke tcb_rest_test(test_string0, "->A[R1]", 21)
        .invoke tcb_rest_test(test_string1, "->C[R3]", 22)
        .invoke tcb_rest_test(test_string2, "->E[R5.'a2 phrase']", 23)
        .invoke tcb_rest_test(test_string3, "->G[R7.'a3 phrase']", 24)
        
        .invoke tcf_phrase_test(test_string0, "", 25)
        .invoke tcf_phrase_test(test_string1, "", 26)
        .invoke tcf_phrase_test(test_string2, "a2 phrase", 27)
        .invoke tcf_phrase_test(test_string3, "a3 phrase", 28)
        .invoke tcb_phrase_test(test_string0, "", 29)
        .invoke tcb_phrase_test(test_string1, "b1 phrase", 30)
        .invoke tcb_phrase_test(test_string2, "", 31)
        .invoke tcb_phrase_test(test_string3, "b3 phrase", 32)
        

        .invoke tcf_kl_test(test_string4, "", 33)
        .invoke tcb_kl_test(test_string4, "", 34)

        .exit 0
        '''
        if rc:
            self.fail(rc)

    
    @evaluate_docstring
    def test_empty_tstrsep_(self, rc):
        '''
        .assign x = ""
        .exit "$tstrsep_{x}"
        '''
        self.assertEqual("",rc)

    @evaluate_docstring
    def test_tstrsep_(self, rc):
        '''
        .assign x = "Hej_Hopp"
        .exit "$tstrsep_{x}"
        '''
        self.assertEqual("Hej",rc)

    @evaluate_docstring
    def test_no_tstrsep_(self, rc):
        '''
        .assign x = "No"
        .exit "$tstrsep_{x}"
        '''
        self.assertEqual("No",rc)

    @evaluate_docstring
    def test_t_strsep(self, rc):
        '''
        .assign x = "Hej_Hopp"
        .exit "$t_strsep{x}"
        '''
        self.assertEqual("Hopp",rc)

    @evaluate_docstring
    def test_no_t_strsep(self, rc):
        '''
        .assign x = "No"
        .exit "$t_strsep{x}"
        '''
        self.assertEqual("",rc)

    @evaluate_docstring
    def test_example_txmlclean(self, rc):
        '''
        .assign x = "ExamplETExt"
        .exit "$txmlclean{x}"
        '''
        self.assertEqual("ExamplETExt", rc)

    @evaluate_docstring
    def test_txmlclean(self, rc):
        '''
        .assign x = "&<>"
        .exit "$txmlclean{x}"
        '''
        self.assertEqual("&amp;&lt;&gt;", rc)

    @evaluate_docstring
    def test_txmlquot(self, rc):
        '''
        .assign x = "B'"
        .assign y = "C"
        .exit "$txmlquot{x} $txmlquot{y}"
        '''
        self.assertEqual(""""B'" 'C'""",rc)

    @evaluate_docstring
    def test_txmlname(self, rc):
        '''
        .assign x1 = "B"
        .assign x2 = "C.9"
        .assign x3 = "D_9"
        .assign x4 = "E-9"
        .assign x5 = ".G"
        .assign x6 = "-H"
        .assign x7 = "_I"
        .exit "$txmlname{x1} $txmlname{x2} $txmlname{x3} $txmlname{x4} $txmlname{x5} $txmlname{x6} $txmlname{x7}"
        '''
        self.assertEqual("B C.9 D_9 E-9 _G _H _I",rc)

    @evaluate_docstring
    def test_tu2d(self, rc):
        '''
        .assign x = "-_"
        .exit "$tu2d{x}"
        '''
        self.assertEqual("--", rc)

    @evaluate_docstring
    def test_td2u(self, rc):
        '''
        .assign x = "-_"
        .exit "$td2u{x}"
        '''
        self.assertEqual("__", rc)

    @evaluate_docstring
    def test_invalid_format(self,rc):
        '''
        .assign x = "-_"
        .exit "$tsomethinginvalid{x}"
        '''
        self.assertIsInstance(rc, RuntimeException)


    @evaluate_docstring
    def test_custom_transform_in_body(self,rc):
        '''
        .function f
          .param string s
Hej $t{s}
        .end function
        .invoke rc = f("Hej!")
        .exit "${rc.body}"
        '''
        self.assertEqual("Hej Hej!\n", rc)

    @evaluate_docstring
    def test_dollar_dollar_in_literal(self,rc):
        '''
        .exit "$${baff}"
        '''
        self.assertEqual("${baff}", rc)

    @evaluate_docstring
    def test_subst_in_subst(self, rc):
        '''
        .assign a = "b"
        .assign b = "Hej"
        .exit "${${a}}"
        '''
        self.assertEqual("b", rc)



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

import unittest

import rsl.symtab


class TestSymbolTable(unittest.TestCase):


    def test_symbol_store(self):
        symtab = rsl.symtab.SymbolTable()
        handle = "TEST"
        
        symtab.enter_scope()
        symtab.enter_scope()
        
        symtab.install_symbol('Test', handle)
        value = symtab.find_symbol("Test")
        symtab.leave_scope()
        
        self.assertRaises(rsl.symtab.SymtabException, symtab.find_symbol, "Test")
        
        self.assertEqual(handle, value)
        
    def test_global_store(self):
        symtab = rsl.symtab.SymbolTable()

        g1 = "Test1"
        g2 = "Test2"
        
        symtab.enter_scope()
        symtab.install_global("g1", g1)
        
        symtab.enter_scope()
        symtab.install_global("g2", g2)
        symtab.leave_scope()

        self.assertEqual(symtab.find_symbol("g1"), g1)
        self.assertEqual(symtab.find_symbol("g2"), g2)
        
        symtab.leave_scope()
        
        self.assertRaises(rsl.symtab.SymtabException, symtab.find_symbol, "g1")
        self.assertRaises(rsl.symtab.SymtabException, symtab.find_symbol, "g2")
        
    def test_out_of_scope(self):
        symtab = rsl.symtab.SymbolTable()
        self.assertRaises(rsl.symtab.SymtabException, symtab.leave_scope)

    def test_out_of_block(self):
        symtab = rsl.symtab.SymbolTable()
        
        symtab.enter_scope()
        symtab.leave_block()
        
        self.assertRaises(rsl.symtab.SymtabException, symtab.leave_block)
        
    def test_symbol_rewrite(self):
        symtab = rsl.symtab.SymbolTable()
        
        symtab.enter_scope()
        
        symtab.install_symbol('TEST', 'TEST1')
        symtab.install_symbol('TEST', 'TEST2')
        
        value = symtab.find_symbol("TEST")
        symtab.leave_scope()
        
        self.assertEqual(value, 'TEST2')
        
    def test_get_scope_symbols(self):
        symtab = rsl.symtab.SymbolTable()
        
        symtab.enter_scope()
        self.assertEqual(len(symtab.scope_head.symbols), 0)
        
        symtab.install_symbol('TEST', 'TEST1')
        symtab.install_symbol('TEST', 'TEST2')
        self.assertEqual(len(symtab.scope_head.symbols), 1)
        
        symtab.install_symbol('TEST1', '')
        self.assertEqual(len(symtab.scope_head.symbols), 2)

        symtab.leave_scope()
        
        symtab.enter_scope()
        self.assertEqual(len(symtab.scope_head.symbols), 0)
        



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2015 John Törnblom

import logging

from setuptools import setup

logging.basicConfig(level=logging.DEBUG)

from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.rst')) as f:
    long_description = f.read()


setup(name='pyrsl',
    version='3.0.0', # ensure that this is the same as in rsl.version
    description='Interpreter for the Rule Specification Language (RSL)',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    author='John Törnblom',
    author_email='john.tornblom@gmail.com',
    url='https://github.com/xtuml/pyrsl',
    license='GPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Interpreters',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 3 :: Only'],
    keywords='rsl xtuml bridgepoint',
    packages=['rsl', 'examples'],
    data_files = [('share/gtksourceview-3.0/language-specs',
                    ['editors/gtksourceview/rsl.lang'])],
    requires=['ply', 'pyxtuml'],
    install_requires=['ply', 'pyxtuml'],
    setup_requires=['ply', 'pyxtuml'])

# assure PLY parstab and lextab get generated
import rsl
rsl.parse_text('')



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring

import rsl


class TestScopeRule(RSLTestCase):

    @evaluate_docstring
    def test_access_global_from_function(self, rc):
        '''
        .assign x = 5
        .function f
            .exit x
        .end function
        .invoke f()
        '''
        self.assertIsInstance(rc, rsl.symtab.SymtabException)
        
    @evaluate_docstring
    def test_recursive_function(self, rc):
        '''
        .function f
          .param integer x
          .assign y = x
          .if (x > 0)
            .invoke res = f(x - 1)
          .end if
          .assign attr_value = y
        .end function
        
        .invoke res = f(3)
        
        .exit res.value
        '''
        self.assertEqual(rc, 3)
        



# encoding: utf-8
# Copyright (C) 2016 John Törnblom
'''
Example on how to extend the functionallity of rsl.gen_erate by defining custom
bridges and string formatters.
'''
import hashlib
import sys

from rsl import gen_erate
from rsl import bridge
from rsl import string_formatter


@bridge('HASH.MD5')
def hash_md5(s):
    try:
        result = hashlib.md5(s).hexdigest()
        success = True
    except:
        result = ''
        success = False

    return {'success': success,
            'result': result}


@string_formatter('trmquot')
def remove_quot(s):
    QUOTES = "'\""
    first_index = 0
    last_index = len(s) - 1
    
    if s[0] in QUOTES:
        first_index += 1

    if s[-1] in QUOTES:
        last_index +- 1

    return s[first_index:last_index]


def main():
    print('Running my custom version of gen_erate')
    rc = gen_erate.main()
    sys.exit(rc)


if __name__ == '__main__':
    main()




# encoding: utf-8
# Copyright (C) 2017 John Törnblom

from utils import RSLTestCase
from utils import lint_docstring


class TestLinting(RSLTestCase):
    
    def setUp(self):
        RSLTestCase.setUp(self)
        self.metamodel.define_class('Cls', [])
        
    @lint_docstring
    def test_empty_text(self, rc):
        ''
        self.assertEqual(0, rc)

    @lint_docstring
    def test_function(self, rc):
        '''
        .function f
        .end function
        '''
        self.assertEqual(0, rc)

    @lint_docstring
    def test_function_duplication(self, rc):
        '''
        .function f
        .end function
        .function f
        .end function
        '''
        self.assertEqual(1, rc)

    @lint_docstring
    def test_unknown_class(self, rc):
        '''
        .create object instance x of Cls
        .select any x from instances of Cls
        .select many x from instances of Cls

        .create object instance x of Missing_Cls
        .select any x from instances of Missing_Cls
        .select many x from instances of Missing_Cls
        '''
        self.assertEqual(3, rc)

    @lint_docstring
    def test_lowercased_class(self, rc):
        '''
        .create object instance x of cls
        '''
        self.assertEqual(1, rc)




# encoding: utf-8
# Copyright (C) 2015 John Törnblom

import sys
import pytest

from utils import RSLTestCase
from utils import evaluate_docstring


class TestPrint(RSLTestCase):

    @evaluate_docstring
    def test_print_string(self, rc):
        '''
        .print "Hello world!"
        '''
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertEqual(output, "test_print.test_print_string: 2:  INFO:  Hello world!")
        
    @evaluate_docstring
    def test_print_substitution_variable(self, rc):
        '''
        .assign s = "world"
        .print "Hello ${s}!"
        '''
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertEqual(output, "test_print.test_print_substitution_variable: 3:  INFO:  Hello world!")
        
    @evaluate_docstring
    def test_print_double_dollar(self, rc):
        '''
        .assign s = "$$world"
        .print "Hello ${s}!"
        '''
        try:
            output = sys.stdout.getvalue().strip()
        except AttributeError:
            s, _ = self.capfd.readouterr()
            output = s.strip()
        self.assertEqual(output, "test_print.test_print_double_dollar: 3:  INFO:  Hello $world!")

    @pytest.fixture(autouse=True)
    def capfd(self, capfd):
        self.capfd = capfd



# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring

from rsl.runtime import RuntimeException


class TestInvoke(RSLTestCase):

    @evaluate_docstring
    def test_invoke_empty_function(self, rc):
        '''
        .function f
        .end function
        .invoke f()
        .exit 1
        '''
        self.assertEqual(1, rc)
        
    @evaluate_docstring
    def test_invoke_dot_named_function(self, rc):
        '''
        .function module.f
        .exit 1
        .end function
        .invoke module.f()
        .exit 0
        '''
        self.assertEqual(1, rc)
    
    @evaluate_docstring
    def test_invoke_with_exit(self, rc):
        '''
        .function f
            .exit 1
        .end function
        .invoke f()
        .exit 0
        '''
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_invoke_with_parameter(self, rc):
        '''
        .function f
            .param integer x
            .exit x + 1
        .end function
        .invoke f(1)
        .exit 0
        '''
        self.assertEqual(2, rc)


    @evaluate_docstring
    def test_parameter_named_from(self, rc):
        '''
        .function f
            .param string from
            .assign from = "f"
            .exit "${from}"
        .end function
        .invoke f("from")
        .exit 0
        '''
        self.assertEqual("f", rc)

    @evaluate_docstring
    def test_parameter_named_cardinality(self, rc):
        '''
        .function f
            .param integer cardinality
            .assign cardinality = (cardinality cardinality)
            .exit "${cardinality}"
        .end function
        .invoke f(3)
        .exit 0
        '''
        self.assertEqual('0', rc)
        
    @evaluate_docstring
    def test_invoke_with_parameter_and_comments(self, rc):
        '''
        .function f .// integer
            .param integer x .// some comment
            .// begin body
            .exit x + 1
        .end function .//
        .invoke f(1)
        .exit 0
        '''
        self.assertEqual(2, rc)
        
    @evaluate_docstring
    def test_invoke_with_comments_between_parameters(self, rc):
        '''
        .function f .// integer
            .param integer x
            .// some comment
            .param integer y
            .// begin body
            .exit x + y
        .end function
        .invoke f(1, 2)
        .exit 0
        '''
        self.assertEqual(3, rc)
        
    @evaluate_docstring
    def test_parameter_order(self, rc):
        '''
        .function f
            .param integer x
            .param integer y
            .param integer z
            .exit x
        .end function
        .invoke f(1, 2, 3)
        .exit 0
        '''
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_missing_parameter(self, rc):
        '''
        .function f
            .param integer x
            .param integer y
            .param integer z
            .exit x
        .end function
        .invoke f(1)
        .exit 0
        '''
        self.assertIsInstance(rc, RuntimeException)

    @evaluate_docstring
    def test_superfluous_argument(self, rc):
        '''
        .function f
            .param integer x
            .param integer y
            .param integer z
            .exit x
        .end function
        .invoke f(1,2,3,4)
        .exit 0
        '''
        self.assertIsInstance(rc, RuntimeException)
        
    @evaluate_docstring
    def test_invoke_with_return_value(self, rc):
        '''
        .function f
            .assign attr_value = 1
        .end function
        .invoke res = f()
        .exit res.value
        '''
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_invoke_dot_named_function_with_return_value(self, rc):
        '''
        .function module.f
            .assign attr_value = 1
        .end function
        .invoke res = module.f()
        .exit res.value
        '''
        self.assertEqual(1, rc)

    @evaluate_docstring
    def test_invoke_with_return_values(self, rc):
        '''
        .function f
            .assign attr_x = 1
            .assign attr_y = 2
        .end function
        .invoke res = f()
        .if (res.x != 1)
            .exit 1
        .end if
        .if (res.y != 2)
            .exit 1
        .end if
        .exit 0
        '''
        self.assertEqual(0, rc)

    @evaluate_docstring
    def test_invoke_from_other_body(self, rc):
        '''
        ..function f
            ..assign attr_x = 1
            ..assign attr_y = 2
        ..end function
        .emit to file "/tmp/RSLTestCase"
        .include "/tmp/RSLTestCase"
        .invoke res = f()
        .if (res.x != 1)
            .exit 1
        .end if
        .if (res.y != 2)
            .exit 1
        .end if
        .exit 0
        '''
        self.assertEqual(0, rc)

    @evaluate_docstring
    def test_invoke_undefined_function(self, rc):
        '''
        .invoke res = f()
        .exit 0
        '''
        self.assertIsInstance(rc, RuntimeException)
        



# encoding: utf-8
# Copyright (C) 2016 John Törnblom

import xtuml

from utils import RSLTestCase


class TestRelate(RSLTestCase):

    def test_relate(self):
        self.metamodel.define_class('A', [('Id', 'unique_id'), ('B_Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id')])
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='A',
                                          target_kind='B',
                                          source_keys=['B_Id'],
                                          target_keys=['Id'],
                                          source_many=False,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=True,
                                          source_phrase='',
                                          target_phrase='')
        text = '''
        .create object instance a of A
        .create object instance b of B
        .relate a to b across R1
        '''
        self.eval_text(text)

        a = self.metamodel.select_any('A')
        b = xtuml.navigate_one(a).B[1]()
        self.assertTrue(b)

    def test_relate_using(self):
        self.metamodel.define_class('Assoc', [('one_side_ID', 'UNIQUE_ID'),
                                              ('other_side_ID', 'UNIQUE_ID')])
        self.metamodel.define_class('Class', [('ID', 'UNIQUE_ID')])
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='Assoc',
                                          target_kind='Class',
                                          source_keys=['one_side_ID'],
                                          target_keys=['ID'],
                                          source_many=True,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=False,
                                          source_phrase='one',
                                          target_phrase='other')
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='Assoc',
                                          target_kind='Class',
                                          source_keys=['other_side_ID'],
                                          target_keys=['ID'],
                                          source_many=True,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=False,
                                          source_phrase='other',
                                          target_phrase='one')
        
        text = '''
        .assign res = 0
        .create object instance cls1 of Class
        .create object instance cls2 of Class
        .create object instance assoc of Assoc 
        .relate cls1 to cls2 across R1.'other' using assoc
        
        .select one assoc related by cls1->Assoc[R1.'other']
        .if (not_empty assoc)
          .assign res = res + 1
        .end if
        
        .select one assoc related by cls2->Assoc[R1.'one']
        .if (not_empty assoc)
          .assign res = res + 1
        .end if
        
        .unrelate cls1 from cls2 across R1.'other' using assoc
        .if (not_empty assoc)
          .assign res = res + 1
        .end if
        
        .select one assoc related by cls1->Assoc[R1.'other']
        .if (empty assoc)
          .assign res = res + 1
        .end if
        
        .select one assoc related by cls2->Assoc[R1.'one']
        .if (empty assoc)
          .assign res = res + 1
        .end if
        
        .exit res
        '''
        res = self.eval_text(text)
        self.assertEqual(res, 5)

    def test_unrelate(self):
        self.metamodel.define_class('A', [('Id', 'unique_id'), ('B_Id', 'unique_id')])
        self.metamodel.define_class('B', [('Id', 'unique_id')])
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='A',
                                          target_kind='B',
                                          source_keys=['B_Id'],
                                          target_keys=['Id'],
                                          source_many=False,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=True,
                                          source_phrase='',
                                          target_phrase='')

        a = self.metamodel.new('A')
        b = self.metamodel.new('B')
        xtuml.relate(a, b, 1)

        text = '''
        .select any a from instances of A
        .select any b from instances of B
        .print "${a}"
        .print "${b}"
        .unrelate a from b across R1
        '''
        self.eval_text(text)
        
        a = self.metamodel.select_any('A')
        b = xtuml.navigate_one(a).B[1]()
        self.assertFalse(b)
        
    def test_relate_reflexive(self):
        self.metamodel.define_class('A', [('Id', 'unique_id'),
                                          ('Next_Id', 'unique_id'),
                                          ('Name', 'string')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='A',
                                          target_kind='A',
                                          source_keys=['Id'],
                                          target_keys=['Next_Id'],
                                          source_many=False,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=True,
                                          source_phrase='prev',
                                          target_phrase='next')
        
        text = '''
        .create object instance a1 of A
        .create object instance a2 of A
        .assign a1.Name = "First"
        .assign a2.Name = "Second"
        .relate a2 to a1 across R1.'next'
        '''
        
        self.eval_text(text)
        a1 = self.metamodel.select_any('A', lambda sel: sel.Name == "First")
        a2 = xtuml.navigate_one(a1).A[1, 'prev']()
        self.assertTrue(a2)
        self.assertEqual(a2.Name, 'Second')

    def test_unrelate_reflexive(self):
        self.metamodel.define_class('A', [('Id', 'unique_id'),
                                          ('Next_Id', 'unique_id'),
                                          ('Name', 'string')])
        
        self.metamodel.define_association(rel_id='R1',
                                          source_kind='A',
                                          target_kind='A',
                                          source_keys=['Id'],
                                          target_keys=['Next_Id'],
                                          source_many=False,
                                          target_many=False,
                                          source_conditional=True,
                                          target_conditional=True,
                                          source_phrase='prev',
                                          target_phrase='next')

        first = self.metamodel.new('A', Name="First")
        second = self.metamodel.new('A', Name="Second")
        xtuml.relate(first, second, 1, 'prev')
        
        text = '''
        .select any first_inst from instances of A where (selected.Name == "First")
        .select one second_inst related by first_inst->A[R1.'next']
        .unrelate first_inst from second_inst across R1.'prev'
        '''
        
        self.eval_text(text)
        a1 = self.metamodel.select_any('A', lambda sel: sel.Name == "First")
        a2 = xtuml.navigate_one(a1).A[1, 'next']()
        self.assertFalse(a2)




# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase
from utils import evaluate_docstring


class TestVariable(RSLTestCase):

    @evaluate_docstring
    def test_variable_with_keyword_name_empty(self, rc):
        '''
        .assign empty = 1
        .exit "${empty}"
        '''
        self.assertEqual("1", rc)
        
    @evaluate_docstring
    def test_variable_with_keyword_name_where(self, rc):
        '''
        .assign where = 1
        .exit "${where}"
        '''
        self.assertEqual("1", rc)

    @evaluate_docstring
    def test_variable_with_keyword_name_in(self, rc):
        '''
        .assign in = 1
        .exit "${in}"
        '''
        self.assertEqual("1", rc)

    @evaluate_docstring
    def test_variable_with_type_name(self, rc):
        '''
        .assign string = 1
        .exit "${string}"
        '''
        self.assertEqual("1", rc)




# encoding: utf-8
# Copyright (C) 2015 John Törnblom

from utils import RSLTestCase

from rsl.runtime import RuntimeException


class TestTypeSystem(RSLTestCase):

    def test_invoke_parameter(self):

        def f(ty, val):
            return self.eval_text('''
                .function f
                    .param %s x
                .end function
                .invoke f(%s)
                .exit true''' % (ty, val), 'TestTypeSys.test_invoke_parameter(%s, %s)' % (ty, val))
        
        self.assertTrue(f('boolean', 'true'))
        self.assertTrue(f('boolean', 'false'))
        self.assertTrue(f('integer', '1'))
        self.assertTrue(f('integer', '-1'))
        self.assertTrue(f('string', '""'))
        self.assertTrue(f('string', '"Hello"'))
        self.assertTrue(f('real', '0.0'))
        self.assertTrue(f('real', '-0.1'))
        self.assertTrue(f('real', '1'))
        
        self.assertIsInstance(f('boolean', '1'), RuntimeException)
        self.assertIsInstance(f('integer', '1.1'), RuntimeException)
        self.assertIsInstance(f('string', 'true'), RuntimeException)
        self.assertIsInstance(f('frag_ref', '0'), RuntimeException)

    def test_invoke_fragment_parameter(self):

        def f(ty):
            return self.eval_text('''
                .function g
                    .assign attr_result = True
                .end function
                .function f
                    .param %s x
                .end function
                .invoke value = g()
                .invoke f(value)
                .exit true''' % ty, 
                'TestTypeSys.test_invoke_fragment_parameter(%s)' % ty)
        
        self.assertTrue(f('frag_ref'))
        
        self.assertIsInstance(f('boolean'), RuntimeException)
        self.assertIsInstance(f('integer'), RuntimeException)
        self.assertIsInstance(f('real'), RuntimeException)
        self.assertIsInstance(f('string'), RuntimeException)

    def test_invoke_unknown_type(self):
        self.metamodel.define_class('My_Class', [('Number', 'integer')])
        inst = self.metamodel.new('My_Class')
        inst.Number = object()
        text = '''
            .function f
                .param integer my_int
                .exit my_int
            .end function
            .select any inst from instances of My_Class
            .invoke res = f(inst.Number)
            .exit -1
        '''
        rc = self.eval_text(text)
        self.assertIsInstance(rc, RuntimeException)
        
    def test_invoke_instref_with_kind(self):
        self.metamodel.define_class('My_Class1', [('Name', 'string')])
        self.metamodel.define_class('My_Class2', [('Name', 'string')])
        self.metamodel.new('My_Class1', Name='1')
        self.metamodel.new('My_Class2', Name='2')
        
        text = '''
            .function f
                .param inst_ref<My_Class1> inst
                .exit 1
            .end function
            .select any inst from instances of My_Class1
            .invoke res = f(inst)
            .exit 0
        '''
        rc = self.eval_text(text)
        self.assertEqual(rc, 1)
        
    def test_invoke_instref_with_wrong_kind(self):
        self.metamodel.define_class('My_Class1', [('Name', 'string')])
        self.metamodel.define_class('My_Class2', [('Name', 'string')])
        self.metamodel.new('My_Class1', Name='1')
        self.metamodel.new('My_Class2', Name='2')
        
        text = '''
            .function f
                .param inst_ref<My_Class2> inst
                .exit 1
            .end function
            .select any inst from instances of My_Class1
            .invoke res = f(inst)
            .exit 0
        '''
        rc = self.eval_text(text)
        self.assertIsInstance(rc, RuntimeException)
        
    def test_invoke_instrefset_with_kind(self):
        self.metamodel.define_class('My_Class1', [('Name', 'string')])
        self.metamodel.define_class('My_Class2', [('Name', 'string')])
        self.metamodel.new('My_Class1', Name='1')
        self.metamodel.new('My_Class2', Name='2')
        
        text = '''
            .function f
                .param inst_ref_set<My_Class1> insts
                .exit 1
            .end function
            .select many insts from instances of My_Class1
            .invoke res = f(insts)
            .exit 0
        '''
        rc = self.eval_text(text)
        self.assertEqual(rc, 1)
        
    def test_invoke_instrefset_with_wrong_kind(self):
        self.metamodel.define_class('My_Class1', [('Name', 'string')])
        self.metamodel.define_class('My_Class2', [('Name', 'string')])
        self.metamodel.new('My_Class1', Name='1')
        self.metamodel.new('My_Class2', Name='2')
        
        text = '''
            .function f
                .param inst_ref_set<My_Class2> insts
                .exit 1
            .end function
            .select many insts from instances of My_Class1
            .invoke res = f(insts)
            .exit 0
        '''
        rc = self.eval_text(text)
        self.assertIsInstance(rc, RuntimeException)
        
        
        
        
